#!/usr/bin/env python3
import sys, argparse, random
# Generate a single testcase for riversystem

parser = argparse.ArgumentParser(description="Generator for riversystem.")
parser.add_argument('-o', metavar='O', type=str, default="random",
                    help='Options for generation (random, flower, yes, impossible, invalid)')
parser.add_argument('-n', metavar='N', type=int, required=True,
                    help='number of vertices')
parser.add_argument('-H', metavar='H', type=int, default=50000000,
                    help='height of tree will be roughly max(logn, n/h)')
parser.add_argument('-l', metavar='L', type=int, default=1,
                    help='number of leaves (only for flower graphs)')
parser.add_argument('-u', metavar='U', type=int, default=0,
                    help='roughly number of unknown vertices')
parser.add_argument('-m', metavar='M', type=int, default=1000000000,
                    help='maximum value of water flows of leaves')
parser.add_argument('-s', metavar='S', type=int, default=None,
                    help='seed for generation (optional)')


# Options:
# flower: will create a flower graph (similar to a star graph). Otherwise a random tree will be created.
# random: will assign valid numbers to all vertices, and just make "u" of them -1.
# onlyleaves: all leaves are known and the others are unknown.
# onlyroot: only the root is known, and it is #leaves.
# yes: will attempt to create a random "yes"-instance.
    # First, a random subset of vertices other than the root become "unknown"
    # The known leaves will get random values.
    # The known non-leaves will get values such that their unknown leaves can be uniquely determined.
# impossible: will ruin a "yes" or "onlyroot"-instance by making it impossible.
# invalid: will ruin a "yes" or "onlyroot"-instance by making it invalid.
# noroot: the root will be made unknown at the end.
# tripleroot: "?-x-?-" is appended to the root.

# Sometimes testcases become "invalid" just because the answers are > 10**9


args = parser.parse_args()
random.seed(args.s)

options = args.o
n = args.n
h = args.H
l = args.l
u = args.u
maxflow = args.m

if "tripleroot" in options:
    n -= 3
    assert(n >= 1)

parent = []
parent.append(-1)

children = [[] for i in range(0,n)]
leaves = []
answer = [-1]*n

if "flower" in options:
    for i in range(0,n-1-l):
        parent.append(i)
    for i in range(0,l):
        parent.append(n-l-1)
else:
    for i in range(0,n-1):
        low = max(0, i-h+1)
        high = i
        parent.append(random.randint(low,high))

for i in range(1,n):
    children[parent[i]].append(i)
for i in range(0,n):
    if len(children[i]) == 0:
        leaves.append(i)

if "onlyleaves" in options:
    for leaf in leaves:
        answer[leaf] = random.randint(1,maxflow)

elif "onlyroot" in options:
    answer[0] = len(leaves)
    if "impossible" in options:
        answer[0] += 1
    if "invalid" in options:
        answer[0] -= 1

elif "random" in options:
    for leaf in leaves:
        answer[leaf] = random.randint(1,maxflow)
    for i2 in range(0,n):
        i = n-i2-1
        if answer[i] == -1:
            answer[i] = sum(answer[j] for j in children[i])
            answer[i] = min(answer[i], 10**9)
    unknown = random.sample(range(0,n), u)
    for i in unknown:
        answer[i] = -1
        

elif "yes" in options:
    unknown = set(random.sample(range(1,n), u))
    unknown_leaves_count = [0]*n
    sum_of_known = [0]*n
    make_impossible = ("impossible" in options)
    make_invalid = ("invalid" in options)

    for i2 in range(0,n):
        i = n-i2-1
        if len(children[i]) == 0:
            if i in unknown:
                unknown_leaves_count[i] = 1
            else:
                answer[i] = random.randint(1,maxflow)
                sum_of_known[i] = answer[i]
        else:
            for c in children[i]:
                if answer[c] == -1:
                    unknown_leaves_count[i] += unknown_leaves_count[c]
                sum_of_known[i] += sum_of_known[c]
            if i not in unknown and sum_of_known[i] < 10**9:
                if unknown_leaves_count[i] == 1:
                    answer[i] = random.randint(sum_of_known[i]+1, min(10**9, sum_of_known[i]+maxflow))
                    sum_of_known[i] = answer[i]
                else:
                    answer[i] = sum_of_known[i] + unknown_leaves_count[i]
                    if make_impossible:
                        make_impossible = False
                        answer[i] += 1
                    elif make_invalid:
                        make_invalid = False
                        answer[i] -= 1
                    answer[i] = min(answer[i], 10**9)
                    answer[i] = max(answer[i], 1)
                    sum_of_known[i] = answer[i]


if "tripleroot" in options:
    for i in range(0,n):
        parent[i] += 3
    parent = [-1,0,1] + parent
    x = answer[0]-1
    x = max(x,-1)
    answer = [-1,x,-1] + answer
    n += 3

if "noroot" in options:
    answer[0] = -1

for i in range(1,n):
    parent[i] += 1
parent.pop(0)
print(n)
print(*parent)

# Change -1 to 0
for i in range(0,n):
    if answer[i] == -1:
        answer[i] = 0
        
print(*answer)
