#!/usr/bin/env python3
import sys, argparse, random
# Generate a single testcase for diplomaceremony

parser = argparse.ArgumentParser(description="Generator for diplomaceremony.")
parser.add_argument('-o', metavar='O', type=str, default="random",
                    help='Options for generation (random, circle, noshuffle)')
parser.add_argument('-n', metavar='N', type=int, required=True,
                    help='number of players')
parser.add_argument('-p', metavar='P', type=int, default=50,
                    help='probability of positive coin flip (0 -- 100)')
parser.add_argument('-s', metavar='S', type=int, default=None,
                    help='seed for generation (optional)')
args = parser.parse_args()
random.seed(args.s)

options = args.o.split('_')
n = args.n
p = args.p
edges = []
sys.stderr.write("Generating case with n=%d, p=%d, options=%s\n" % (n, p, options))

if "circle" in options:
    is_played = [[False] * n for _ in range(n)]
    for i in range(n):
        is_played[i][i] = True
        
    for i in range(n):
        for j in range(i+1, i+(n+1)//2):
            if not is_played[i][j % n]:
                edges.append((i, j % n))
                is_played[i][j % n] = True
                is_played[j % n][i] = True
            else:
                sys.stderr.write(f"Circle active, n = {n}, match already played: {i} vs {j}; will skip\n")
                
    for i in range(n):
        for j in range(i + 1, n):
            if not is_played[i][j]:
                # sys.stderr.write(f"Circle active, n = {n}, non-played match: {i} vs {j}; will generate a result randomly\n")
                flip = random.randrange(0, 100)
                if flip < p:
                    edges.append((i, j))
                else:
                    edges.append((j, i))

else:
    # Random
    for i in range(n):
        for j in range(i + 1, n):
            flip = random.randrange(0, 100)
            if flip < p:
                edges.append((i, j))
            else:
                edges.append((j, i))
    

mask = list(range(n))
if "noshuffle" not in options:
    random.shuffle(edges)
    random.shuffle(mask)    

edges = [(mask[e[0]], mask[e[1]]) for e in edges]

# Convert to matrix
matrix = [["0"]*(i+1) for i in range(n-1)]
# print(matrix)
for a, b in edges:
    if a < b:
        # Latest ID lost
        matrix[b-1][a] = "0"
    else:
        matrix[a-1][b] = "1"


print(n)
for line in matrix:
    print("".join(line))

